import threading
from time import sleep

from PyQt5.QtQml import QQmlApplicationEngine
from PyQt5.QtQuick import QQuickItem

class Video():


    """
    Provides interface between qml SVideo and QVideo
    """

    def __init__(self, engine=None):
        self._engine = engine
        self._root_object = self._engine.rootObjects()[0]
        self.QVideo = ()

    def __str__(self):
        return 'soloman.Video'

    def get_SVideo(self, obj_name):
        g_thread = threading.Thread(target=self._get_SVideo, args=[obj_name])
        g_thread.daemon=True
        g_thread.start()

    def show_frame(self, frame):
        g_thread = threading.Thread(target=self._show_frame, args=[frame])
        g_thread.daemon=True
        g_thread.start()

    def _show_frame(self, frame):
        self.QVideo.show_cv2_frame(frame)

    def _get_SVideo(self, obj_name):
        """
        Get the SVideo's QVideo object
        """
        self.QVideo = self._root_object.findChild(QQuickItem, obj_name)
        self.QVideo.start_cv2()

    def update(self, frame):
        g_thread = threading.Thread(target=self._update, args=[frame])
        g_thread.daemon=True
        g_thread.start()

    def _update(self, frame):
        """
        Updates the QVideo frames
        """
        self.QVideo.make_cv2_frame(frame)



"""
This module contains the helper functions for opencv users
so their application will loop not less than the needed time
for the various fps rates
The sleep should happen faster than the fps rate
"""
from time import sleep

def fps_24():
    """
    now set at 1/40
    """
    sleep(1/40)
    return True

def fps_30():
    """
    now set at 1/60
    """
    sleep(1/60)
    return True

def fps_60():
    """
    now set at 1/120
    """
    sleep(1/120)
    return True



# -*- coding: utf-8 -*-
"""
Created on Tue Mar 31 2020 12:50:44

"""
import os
from time import sleep

from PyQt5.QtQml import qmlRegisterType

# import Audio and Video here so people can import it without
# going into the audio and video packages to access them
from .audio import Audio
from .video import Video
from soloman.pyqt_inter_audio import QAudio
from soloman.pyqt_inter_video import QVideo
from soloman.fps_clocks import *


def register():

    # Delete the contents of our Temp folder
    path = os.path.join(os.environ['TEMP'], 'soloman', 'converts')
    if os.path.exists(path):
        os.system('RD "' + path + '" /S /Q')

    qmlRegisterType(QAudio, 'soloman', 2, 5, 'SAudio')
    qmlRegisterType(QVideo, 'soloman', 2, 5, 'QVideo')


register()



# -*- coding: utf-8 -*-
"""
Created on Fri Apr  3 21:33:24 2020

@author: Ampofo
"""
import os
import threading

from PyQt5.QtCore import pyqtProperty, QObject, pyqtSlot

from .audio import Audio


class QAudio(QObject):


    """
    """

    def __init__(self, parent=None):
        super().__init__(parent)
        self.aud = Audio()

        self._save_folder = ''
        self._volume_level = 1.4

    def _delay_play(self, u_delay):
        self.aud.delay_play(u_delay)

    def _pause(self):
        self.aud.pause()

    def _play(self, filename):
        filename = os.path.realpath(filename)
        self.aud.play(filename)

    def _prepare(self, filename):
        filename = os.path.realpath(filename)
        self.aud.prepare(filename)

    def _resume(self):
        self.aud.resume()

    def _seek(self, seconds):
        self.aud.seek(seconds)

    def _stop(self):
        self.aud.stop()

    def _t_played(self):
        self.aud.t_played()

    @pyqtSlot(int)
    def delay_play(self, u_delay):
        delay_thread = threading.Thread(target=self._delay_play, args=[u_delay])
        delay_thread.daemon = True
        delay_thread.start()

    @pyqtSlot()
    def pause(self):
        pause_thread = threading.Thread(target=self._pause)
        pause_thread.daemon = True
        pause_thread.start()

    @pyqtSlot(str)
    def play(self, filename):
        play_thread = threading.Thread(target=self._play, args=[filename])
        play_thread.daemon = True
        play_thread.start()

    @pyqtSlot(str)
    def prepare(self, filename):
        prepare_thread = threading.Thread(target=self._prepare, args=[filename])
        prepare_thread.daemon = True
        prepare_thread.start()

    @pyqtSlot()
    def resume(self):
        resume_thread = threading.Thread(target=self._resume)
        resume_thread.daemon = True
        resume_thread.start()

    @pyqtProperty('QString')
    def saveFolder(self):
        return self._save_folder

    @saveFolder.setter
    def saveFolder(self, folder_name):
        self._save_folder = os.path.realpath(folder_name)
        self.aud.save_folder = self._save_folder

    @pyqtSlot(int)
    def seek(self, seconds):
        seek_thread = threading.Thread(target=self._seek, args=[seconds])
        seek_thread.daemon = True
        seek_thread.start()

    @pyqtSlot()
    def stop(self):
        stop_thread = threading.Thread(target=self._stop)
        stop_thread.daemon = True
        stop_thread.start()

    @pyqtSlot()
    def t_played(self):
        t_thread = threading.Thread(target=self._t_played)
        t_thread.daemon = True
        t_thread.start()

    @pyqtProperty(int)
    def volume(self):
        return self._volume_level

    @volume.setter
    def volume(self, level):
        self._volume_level = level
        self.aud.controlVolume(self._volume_level)



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages

with open('README.md', 'r') as rm:
    long_desc = rm.read()

setup(
    name='soloman',
    version='2.5a0',
    description='For the love of python and qml',
    long_description=long_desc,
    long_description_content_type='text/markdown',
    install_requires=['numpy', 'pyaudio', 'pyffmpeg'],
    keywords="audio, pyaudio, ffmpeg, qml, pyffmpeg",
    url='https://github.com/deuteronomy-works/soloman',
    author='Amoh - Gyebi Godwin Ampofo Michael',
    author_email='amohgyebigodwin@gmail.com',
    project_urls={
        "Bug Tracker": "https://github.com/deuteronomy-works/soloman/issues/",
        "Documentation": "https://github.com/deuteronomy-works/soloman/wiki/",
        "Source Code": "https://github.com/deuteronomy-works/soloman/",
    },
    classifiers = [
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Development Status :: 5 - Production/Stable",
        "Environment :: Other Environment",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Software Development :: Libraries :: Python Modules"
    ],
    packages=['soloman', 'soloman.audio', 'soloman.video', 'PyQt5'],
    package_data={'PyQt5': ['Qt/qml/soloman/qmldir', 'Qt/qml/soloman/*.qml']},
)
